from .editor import PresentationSectionType, get_config, set_config
from .config import Config

set_config(Config)

# NORMAL = PresentationSectionType.NORMAL
# SKIP = PresentationSectionType.SKIP
# LOOP = PresentationSectionType.LOOP
# COMPLETE_LOOP = PresentationSectionType.COMPLETE_LOOP
